# Standard library imports
import os
import pathlib
from datetime import datetime
from pathlib import Path
from math import asin, cos, radians, sin, sqrt
from tkinter import messagebox

# Third party imports
import matplotlib.pyplot as plt
import numpy as np
import pandas as pd

# Internal imports
from ctg.ctgfuncts.ctg_tools import built_lat_long

class EffectifCtg():


    def __init__(self,year:str,ctg_path:pathlib.WindowsPath):

        # get effectif of the year year
        self.year = year
        self.ctg_path = ctg_path
        path_root = self.ctg_path / Path(str(year))/ Path('DATA')
        df = pd.read_excel(path_root / Path(str(year)+'.xlsx'))

        # get effectif of the year year-1
        year_1 = int(year)-1
        path_root = self.ctg_path / Path(str(year_1))/ Path('DATA')
        df_1 = pd.read_excel(path_root /Path(str(year_1)+'.xlsx'))

        # add column Age compute at the 30 september of year
        df['Date de naissance'] = pd.to_datetime(df['Date de naissance'],
                                                 format="%d/%m/%Y")
        df_1['Date de naissance'] = pd.to_datetime(df_1['Date de naissance'],
                                                   format="%d/%m/%Y")

        df['Age']  = df['Date de naissance'].apply(lambda x :
                                                   (pd.Timestamp(int(year), 9, 30)-x).days/365)
        df_1['Age']  = df_1['Date de naissance'].apply(lambda x :
                                                      (pd.Timestamp(int(year), 9, 30)-x).days/365)

        # add column distance from Grenoble
        dh = built_lat_long(df)

        df['distance'] = df.apply(lambda row: self.distance_(row, dh),axis=1)

        self.effectif = df      # effectif year
        self.effectif_1 = df_1  # effectif year moins un

        # deal with nouveaux entrants and sortants
        (self.moy_age_entrants,
        self.nbr_nouveaux_membres,
        self.nouveaux_membres_noms) = self.nouveaux_entrants()
        self.moy_age_sortants, self.nbr_sortants, self.sortants_noms = self.sortants()
        self.rebond = self.get_rebond()

        self.cotisation_licence,self.cotisation_totale, self.cotisation_ctg = self.cotisation()

        self.membres_sympathisants, self.nbr_membres_sympathisants = self.membres_sympathisants()

    @staticmethod
    def distance_(row,dh):

        phi1, lon1 = dh.query("Ville=='GRENOBLE'")[['long','lat']].values.flatten()
        phi1, lon1 = radians(phi1), radians(lon1)
        phi2, lon2 = radians(row['long']), radians(row['lat'])
        rad = 6371
        dist = 2 * rad * asin(sqrt(sin((phi2 - phi1) / 2) ** 2
                                 + cos(phi1) * cos(phi2) * sin((lon2 - lon1) / 2) ** 2))
        return np.round(dist,1)

    def stat(self):

        da = self.effectif.groupby('Sexe')['Age'].agg(['count','median','max','min'])
        res = self.effectif['Age'].agg(['count','median','max','min']).tolist()
        stat = []
        stat.append(f"Année : {self.year}")
        stat.append(" ")
        nbr_membres = round(res[0],0)
        stat.append(f"Nombre d'adhérents : {nbr_membres}")
        nbr_femmes = da.loc['F','count']
        stat.append(f"Nombre de femmes : {nbr_femmes} ({round(100*nbr_femmes/nbr_membres,1)} %)")
        nbr_hommes = da.loc['M','count']
        stat.append(f"Nombre d'hommes : {nbr_hommes} ({round(100*nbr_hommes/nbr_membres,1)} %)")
        stat.append(' ')
        stat.append(f"Age médian total : {round(res[1],1)} ans")
        stat.append(f"Age maximum : {round(res[2],1)} ans")
        stat.append(f"Age minimum : {round(res[3],1)} ans")
        stat.append(f"Age médian des femmes : {round(da.loc['F','median'],1)} ans")
        stat.append(f"Age médian des hommes : {round(da.loc['M','median'],1)} ans")
        stat.append(f"Age maximum des femmes : {round(da.loc['F','max'],1)} ans")
        stat.append(f"Age maximum des hommes : {round(da.loc['M','max'],1)} ans")
        stat.append(f"Age minimum des femmes : {round(da.loc['F','min'],1)} ans")
        stat.append(f"Age minimum des hommes : {round(da.loc['M','min'],1)} ans")
        stat.append(' ')


        stat.append(f'Nombre de membres sympatisant : {self.nbr_membres_sympathisants}')
        stat.append(f'Membres sympatisants : {self.membres_sympathisants}')
        stat.append(' ')

        long_str = (f"{self.nbr_nouveaux_membres} "
                     "nouveaux membres de moyenne d'âge de "
                    f"{round(self.moy_age_entrants,1)} ans")
        stat.append(long_str)
        stat.append(f"Liste des nouveaux :\n{self.nouveaux_membres_noms}")
        long_str = (f"{self.nbr_sortants} "
                     "licences non renouvellées de moyenne d'âge de "
                    f"{round(self.moy_age_sortants,1)} ans")
        stat.append(long_str)
        stat.append(f"Liste des sortants :\n{self.sortants_noms}")
        stat.append(f'Nombre de rebonds : {len(self.rebond)}')
        rebond_str = '; '.join(self.rebond)
        stat.append(f'Membres rebonds : {rebond_str}')
        stat.append(' ')

        if 'Pratique VAE' in self.effectif.columns:
            da = self.effectif.groupby(['Sexe','Pratique VAE'])['Nom'].agg(['count'])
            nbr_vae_femme = nbr_femmes-da.loc['F','Non']['count']
            nbr_vae_homme = nbr_hommes-da.loc['M','Non']['count']
            nbr_vae_tot = nbr_vae_femme + nbr_vae_homme
            long_string = ("Nombre de membres équippées de VAE : "
                          f"{nbr_vae_tot} ({round(100*nbr_vae_tot/nbr_membres,1)} %)")
            stat.append(long_string)
            long_string = ("Nombre de femmes équippées de VAE : "
                          f"{nbr_vae_femme} ({round(100*nbr_vae_femme/nbr_femmes,1)} %)")
            stat.append(long_string)
            long_string = ("Nombre d'hommes équippés de VAE: "
                          f"{nbr_vae_homme} ({round(100*nbr_vae_homme/nbr_hommes)} %)")
            stat.append(long_string)

        stat.append(' ')
        da = self.effectif.groupby(['Discipline'])['Nom'].agg('count')
        for pratique in da.index:
            stat.append(f'{pratique} : {da[pratique]}')
        stat.append(' ')

        self.effectif = self.effectif.rename(columns={'\n\t\t\t\tAbonnements':'Abonnements'})
        nbr_abonnements = len(self.effectif.query('Abonnements == "Oui"'))
        long_string = (f"Nombre d'abonnés à la revue FFCT : {nbr_abonnements} "
                       f"({round(100*nbr_abonnements/nbr_membres)} %)")
        stat.append(long_string)
        stat.append(f"\ncotisation licence ffct : {self.cotisation_licence} €")
        stat.append(f"cotisation totale : {self.cotisation_totale} €")
        stat.append(f"cotisation CTG : {self.cotisation_ctg} €")
        
        path_root = self.ctg_path / Path(str(self.year)) / Path('STATISTIQUES') / Path('TEXT')
        path_root = path_root / Path(f'info_effectif_{self.year}.txt')
        with open(path_root,'w') as f:
            f.write('\n'.join(stat))

        stat.append(("\n Ces informations sont disponibles dans le fichier : "
                    f"\n{path_root}"))
        stat ='\n'.join(stat)
        messagebox.showinfo(f'Statistique {self.year}',stat)


        

    def nouveaux_entrants(self):
        nouveaux_membres_id = set(self.effectif["N° Licencié"])- \
                                set(self.effectif_1["N° Licencié"])


        dg = self.effectif[self.effectif['N° Licencié'].isin(nouveaux_membres_id)]
        moy_age_entrants = dg['Age'].mean() + 1

        nouveaux_membres_list = []
        for idx,row in dg.iterrows():
            nouveaux_membres_list.append(f"{row['Prénom'][0]}. {row['Nom']}")
        nouveaux_membres_noms = '; '.join(nouveaux_membres_list)
        nbr_nouveaux_membres = len(dg)

        return moy_age_entrants, nbr_nouveaux_membres, nouveaux_membres_noms

    def membres_sympathisants(self):
        path_root = self.ctg_path / Path(str(self.year))/Path('DATA')
        file_path = path_root / Path('membres_sympatisants.xlsx')
        if os.path.isfile(file_path):
            membres_sympathisants_df = pd.read_excel(file_path)
            membres_sympathisants_df['Nom_Prenom'] = membres_sympathisants_df['Nom']+\
                                                     " "+\
                                                     membres_sympathisants_df['Prénom'].str[0]
            membres_sympathisants = ', '.join(membres_sympathisants_df['Nom_Prenom'].tolist())
            nbr_membres_sympathisants = len(membres_sympathisants_df)
        else:
            nbr_membres_sympathisants = 'inconnu'
            membres_sympathisants = 'inconnu'

        return membres_sympathisants, nbr_membres_sympathisants

    def sortants(self):
        sortants_id = set(self.effectif_1["N° Licencié"]) - set(self.effectif["N° Licencié"])

        dg = self.effectif_1[self.effectif_1['N° Licencié'].isin(sortants_id)]

        moy_age_sortants = dg['Age'].mean() + 1

        sortants_list = []
        for idx,row in dg.iterrows():
            sortants_list.append(f"{row['Prénom'][0]}. {row['Nom']}")
        sortants_noms = '; '.join(sortants_list)
        nbr_sortants = len(dg)

        return moy_age_sortants, nbr_sortants, sortants_noms

    def cotisation(self):
        cotisation_licence = 'inconnue'
        if 'Cotisation Licence' in self.effectif.columns:
            cotisation_licence = sum(self.effectif['Cotisation Licence'])

        cotisation_totale = 'inconnue'
        if 'Cotisation Totale' in self.effectif.columns:
            cotisation_totale = sum(self.effectif['Cotisation Totale'])

        cotisation_ctg = 15*len(self.effectif)
        return cotisation_licence,cotisation_totale, cotisation_ctg

    def get_rebond(self):
        current_year = datetime.now().year
        path_history = self.ctg_path / Path(str(current_year))
        path_history = path_history / Path('STATISTIQUES/EXCEL/effectif_history.xlsx')
        df = pd.read_excel(path_history)
        df = df.fillna(0)

        year_int = int(self.year)
        list_rebond = []
        for idx,row in df.iterrows():
            if (row[year_int-2] == 0 and row[year_int-1] == year_int-1 and row[year_int] ==0):
                list_rebond.append('-'.join([row['Nom'],row['Prénom']]))
        return list_rebond

    def plot_histo(self):

        fig, ax = plt.subplots(figsize=(10,10))
        self.effectif['age group'] = pd.cut(self.effectif.Age, bins=range(0, 95, 5), right=False)
        result_hist = self.effectif.groupby('Sexe')['age group']
        result_hist = result_hist.value_counts().unstack().T
        result_hist = result_hist.plot.bar(width=1, stacked=False,ax=ax)

        plt.tick_params(axis='x', labelsize=20)
        plt.tick_params(axis='y', labelsize=20)
        plt.title(self.year,fontsize=20)
        plt.legend()
        plt.tight_layout()
        plt.show()



__all__ = ['create_tendance_analysis']

# Standard library imports
import os
import shutil
import tkinter as tk
from tkinter import font as tkFont
from tkinter import filedialog
from tkinter import messagebox
from pathlib import Path

# 3rd party imports
import pandas as pd

# Internal imports
import ctg.ctggui.guiglobals as gg
from ctg.ctgfuncts.ctg_effectif import statistique_vae
from ctg.ctgfuncts.ctg_effectif import builds_excel_presence_au_club
from ctg.ctgfuncts.ctg_effectif import anciennete_au_club
from ctg.ctggui.guitools import encadre_RL
from ctg.ctggui.guitools import font_size
from ctg.ctggui.guitools import mm_to_px
from ctg.ctggui.guitools import place_after
from ctg.ctggui.guitools import place_bellow
from ctg.ctggui.guitools import last_available_years
from ctg.ctgfuncts.ctg_synthese import evolution_sorties

def create_tendance_analysis(self, master, page_name, institute, ctg_path):

    """
    Description : function working as a bridge between the BiblioMeter
    App and the functionalities needed for the use of the app

    Uses the following globals :
    - DIC_OUT_PARSING
    - FOLDER_NAMES

    Args: takes only self and ctg_path as arguments.
    self is the instense in which PageThree will be created
    ctg_path is a type Path, and is the path to where the folders
    organised in a very specific way are stored

    Returns : nothing, it create the page in self
    """

    # Internal functions
    def _launch_evolution_sorties(ctg_path):

        # Getting year selection
        activite =  variable_activite.get()

        evolution_sorties (activite,ctg_path)

    def _launch_member_history_analysis(ctg_path):

        # Getting year selection

        out_path = builds_excel_presence_au_club(ctg_path)

        info_title = "- Information -"
        info_text  = ("L'analyse de la présence au club par membre a été effectuée  "
                      "\n\nLe fichier EXCEL obtenu synthese_adherent.xlsx "
                      "a été créé dans le dossier :"
                     f"\n\n{out_path}.")

        messagebox.showinfo(info_title, info_text)

        anciennete_au_club(ctg_path)

    def _launch_vae_analysis(ctg_path):
        statistique_vae(ctg_path)

    from ctg.ctggui.pageclasses import AppMain

    # Setting effective font sizes and positions (numbers are reference values)
    eff_etape_font_size      = font_size(gg.REF_ETAPE_FONT_SIZE,   AppMain.width_sf_min)
    eff_launch_font_size     = font_size(gg.REF_ETAPE_FONT_SIZE-1, AppMain.width_sf_min)
    eff_help_font_size       = font_size(gg.REF_ETAPE_FONT_SIZE-2, AppMain.width_sf_min)
    eff_select_font_size     = font_size(gg.REF_ETAPE_FONT_SIZE, AppMain.width_sf_min)
    eff_buttons_font_size    = font_size(gg.REF_ETAPE_FONT_SIZE-3, AppMain.width_sf_min)

    synthese_analysis_x_pos_px     = mm_to_px(10 * AppMain.width_sf_mm,  gg.PPI)
    synthese_analysis_y_pos_px     = mm_to_px(40 * AppMain.height_sf_mm, gg.PPI)
    year_analysis_label_dx_px  = mm_to_px( 0 * AppMain.width_sf_mm,  gg.PPI)
    year_analysis_label_dy_px  = mm_to_px(15 * AppMain.height_sf_mm, gg.PPI)
    launch_dx_px             = mm_to_px( 0 * AppMain.width_sf_mm,  gg.PPI)
    launch_dy_px             = mm_to_px( 5 * AppMain.height_sf_mm, gg.PPI)

    year_button_x_pos        = mm_to_px(gg.REF_YEAR_BUT_POS_X_MM * AppMain.width_sf_mm,  gg.PPI)
    year_button_y_pos        = mm_to_px(gg.REF_YEAR_BUT_POS_Y_MM * AppMain.height_sf_mm, gg.PPI)
    dy_year                  = -6
    ds_year                  = 5

    # Setting common attributs
    etape_label_format = 'left'
    etape_underline    = -1

    ### Choix de l'année
    list_activites = gg.ACTIVITE_LIST + ['synthese']
    default_activite = list_activites[-1]
    variable_activite = tk.StringVar(self)
    variable_activite.set(default_activite)

        # Création de l'option button des activités
    self.font_OptionButton_years = tkFont.Font(family = gg.FONT_NAME,
                                               size = eff_buttons_font_size)
    self.OptionButton_years = tk.OptionMenu(self,
                                            variable_activite,
                                            *list_activites)
    self.OptionButton_years.config(font = self.font_OptionButton_years)

        # Création du label
    self.font_Label_years = tkFont.Font(family = gg.FONT_NAME,
                                        size = eff_select_font_size,
                                        weight = 'bold')
    self.Label_years = tk.Label(self,
                                text = gg.TEXT_ACTIVITE_PI,
                                font = self.font_Label_years)
    self.Label_years.place(x = year_button_x_pos, y = year_button_y_pos)

    place_after(self.Label_years, self.OptionButton_years, dy = dy_year)

    ################## Synthèse des activités

    ### Titre
    synthese_analysis_font = tkFont.Font(family = gg.FONT_NAME,
                                         size = eff_etape_font_size,
                                         weight = 'bold')
    synthese_analysis_label = tk.Label(self,
                                       text = gg.TEXT_TENDANCE_SORTIES,
                                       justify = etape_label_format,
                                       font = synthese_analysis_font,
                                       underline = etape_underline)

    synthese_analysis_label.place(x = synthese_analysis_x_pos_px,
                                  y = synthese_analysis_y_pos_px)

    ### Explication
    help_label_font = tkFont.Font(family = gg.FONT_NAME,
                                  size = eff_help_font_size)
    help_label = tk.Label(self,
                          text = gg.HELP_TENDANCE_SORTIES,
                          justify = "left",
                          font = help_label_font)
    place_bellow(synthese_analysis_label,
                 help_label)

    ### Bouton pour lancer l'analyse tendancielle des activités
    synthese_analysis_launch_font = tkFont.Font(family = gg.FONT_NAME,
                                                size = eff_launch_font_size)
    synthese_analysis_launch_button = tk.Button(self,
                                                text = gg.BUTT_TENDANCE_SORTIES,
                                                font = synthese_analysis_launch_font,
                                                command = lambda: _launch_evolution_sorties(ctg_path))
    place_bellow(help_label,
                 synthese_analysis_launch_button,
                 dx = launch_dx_px,
                 dy = launch_dy_px)

    ################## Analyse de de la présence des membres au club

    ### Titre
    member_analysis_label_font = tkFont.Font(family = gg.FONT_NAME,
                                             size = eff_etape_font_size,
                                             weight = 'bold')
    member_analysis_label = tk.Label(self,
                                     text = gg.TEXT_PRESENCE_EFFECTIF,
                                     justify = "left",
                                     font = member_analysis_label_font)
    place_bellow(synthese_analysis_launch_button,
                 member_analysis_label,
                 dx = year_analysis_label_dx_px,
                 dy = year_analysis_label_dy_px)

    ### Explication de l'étape
    help_label_font = tkFont.Font(family = gg.FONT_NAME,
                                  size = eff_help_font_size)
    help_label = tk.Label(self,
                          text = gg.HELP_PRESENCE_EFFECTIF,
                          justify = "left",
                          font = help_label_font)
    place_bellow(member_analysis_label,
                 help_label)

    ### Bouton pour lancer l'analyse des mots clefs
    member_analysis_launch_font = tkFont.Font(family = gg.FONT_NAME,
                                                size = eff_launch_font_size)
    member_analysis_button = tk.Button(self,
                                         text = gg.BUTT_PRESENCE_EFFECTIF,
                                         font = member_analysis_launch_font,
                                         command = lambda: _launch_member_history_analysis(ctg_path))
    place_bellow(help_label,
                 member_analysis_button,
                 dx = launch_dx_px,
                 dy = launch_dy_px)

################## Analyse de l'évolution de la population VAE

    ## Titre
    vae_analysis_label_font = tkFont.Font(family = gg.FONT_NAME,
                                          size = eff_etape_font_size,
                                          weight = 'bold')
    vae_analysis_label = tk.Label(self,
                                  text = gg.TEXT_VAE_ANALYSIS,
                                  justify = "left",
                                  font = vae_analysis_label_font)
    place_bellow(member_analysis_button,
                 vae_analysis_label,
                 dx = year_analysis_label_dx_px,
                 dy = year_analysis_label_dy_px)

    ### Explication de l'étape
    help_label_font = tkFont.Font(family = gg.FONT_NAME,
                                  size = eff_help_font_size)
    help_label_vae = tk.Label(self,
                              text = gg.HELP_VAE_ANALYSIS,
                              justify = "left",
                              font = help_label_font)
    place_bellow(vae_analysis_label,
                 help_label_vae)

    ### Bouton pour lancer l'analyse evolution VAE
    vae_analysis_launch_font = tkFont.Font(family = gg.FONT_NAME,
                                           size = eff_launch_font_size)
    vae_analysis_button = tk.Button(self,
                                    text = gg.BUTT_VAE_ANALYSIS,
                                    font = vae_analysis_launch_font,
                                    command = lambda: _launch_vae_analysis(ctg_path))
    place_bellow(help_label_vae,
                 vae_analysis_button,
                 dx = launch_dx_px,
                 dy = launch_dy_px)


__all__ = ['create_synthese_analysis']

# Standard library imports
import os
import shutil
import tkinter as tk
from tkinter import font as tkFont
from tkinter import filedialog
from tkinter import messagebox
from pathlib import Path

# 3rd party imports
import pandas as pd

# Local imports
import ctg.ctggui.guiglobals as gg
from ctg.ctggui.guitools import encadre_RL
from ctg.ctggui.guitools import font_size
from ctg.ctggui.guitools import mm_to_px
from ctg.ctggui.guitools import place_after
from ctg.ctggui.guitools import place_bellow
from ctg.ctggui.guitools import last_available_years
from ctg.ctgfuncts.ctg_synthese import synthese
from ctg.ctgfuncts.ctg_synthese import plot_pie_synthese
from ctg.ctgfuncts.ctg_synthese import synthese_adherent

def create_synthese_analysis(self, master, page_name, institute, ctg_path):

    """
    Description : function working as a bridge between the BiblioMeter
    App and the functionalities needed for the use of the app

    Uses the following globals :
    - DIC_OUT_PARSING
    - FOLDER_NAMES

    Args: takes only self and ctg_path as arguments.
    self is the instense in which PageThree will be created
    ctg_path is a type Path, and is the path to where the folders
    organised in a very specific way are stored

    Returns : nothing, it create the page in self
    """

    # Internal functions
    def _launch_synthese_analysis():

        # Getting year selection
        year_select =  variable_years.get()
        method = variable_methods.get()
        mode = False if method=='jours' else True
        synthese(year_select,ctg_path)
        plot_pie_synthese(year_select,ctg_path,mode)

    def _launch_member_analysis(ctg_path):

        # Getting year selection
        year_select =  variable_years.get()

        synthese_adherent(year_select,ctg_path)
        info_title = "- Information -"
        info_text  = ("L'analyse de la participation aux sorties par membre "
                     f"a été effectuée pour l'année {year_select} "
                     f"\n\nLe fichier EXCEL obtenu : 'synthese_adherent.xlsx' "
                      "a été créé dans le dossier :"
                     f"\n\n{ctg_path}\{str(year_select)}\STATISTIQUES\EXCEL.")

        messagebox.showinfo(info_title, info_text)

    from ctg.ctggui.pageclasses import AppMain

    # Setting effective font sizes and positions (numbers are reference values)
    eff_etape_font_size      = font_size(gg.REF_ETAPE_FONT_SIZE,   AppMain.width_sf_min)
    eff_launch_font_size     = font_size(gg.REF_ETAPE_FONT_SIZE-1, AppMain.width_sf_min)
    eff_help_font_size       = font_size(gg.REF_ETAPE_FONT_SIZE-2, AppMain.width_sf_min)
    eff_select_font_size     = font_size(gg.REF_ETAPE_FONT_SIZE, AppMain.width_sf_min)
    eff_buttons_font_size    = font_size(gg.REF_ETAPE_FONT_SIZE-3, AppMain.width_sf_min)

    synthese_analysis_x_pos_px     = mm_to_px(10 * AppMain.width_sf_mm,  gg.PPI)
    synthese_analysis_y_pos_px     = mm_to_px(40 * AppMain.height_sf_mm, gg.PPI)
    year_analysis_label_dx_px  = mm_to_px( 0 * AppMain.width_sf_mm,  gg.PPI)
    year_analysis_label_dy_px  = mm_to_px(15 * AppMain.height_sf_mm, gg.PPI)
    launch_dx_px             = mm_to_px( 0 * AppMain.width_sf_mm,  gg.PPI)
    launch_dy_px             = mm_to_px( 5 * AppMain.height_sf_mm, gg.PPI)

    year_button_x_pos        = mm_to_px(gg.REF_YEAR_BUT_POS_X_MM * AppMain.width_sf_mm,  gg.PPI)
    year_button_y_pos        = mm_to_px(gg.REF_YEAR_BUT_POS_Y_MM * AppMain.height_sf_mm, gg.PPI)
    dy_year                  = -6
    ds_year                  = 5

    # Setting common attributs
    etape_label_format = 'left'
    etape_underline    = -1

    ### Choix de l'année
    list_years = last_available_years(ctg_path,year_number=2020)[1:]
    default_year = list_years[-1]
    variable_years = tk.StringVar(self)
    variable_years.set(default_year)

        # Création de l'option button des années
    self.font_OptionButton_years = tkFont.Font(family = gg.FONT_NAME,
                                               size = eff_buttons_font_size)
    self.OptionButton_years = tk.OptionMenu(self,
                                            variable_years,
                                            *list_years)
    self.OptionButton_years.config(font = self.font_OptionButton_years)

        # Création du label
    self.font_Label_years = tkFont.Font(family = gg.FONT_NAME,
                                        size = eff_select_font_size,
                                        weight = 'bold')
    self.Label_years = tk.Label(self,
                                text = gg.TEXT_YEAR_PI,
                                font = self.font_Label_years)
    self.Label_years.place(x = year_button_x_pos, y = year_button_y_pos)

    place_after(self.Label_years, self.OptionButton_years, dy = dy_year)
    

    ################## Synthèse des sorties

    ### Titre
    synthese_analysis_font = tkFont.Font(family = gg.FONT_NAME,
                                         size = eff_etape_font_size,
                                         weight = 'bold')
    synthese_analysis_label = tk.Label(self,
                                       text = gg.TEXT_SYNTHESE_SORTIES,
                                       justify = etape_label_format,
                                       font = synthese_analysis_font,
                                       underline = etape_underline)

    synthese_analysis_label.place(x = synthese_analysis_x_pos_px,
                                  y = synthese_analysis_y_pos_px)

    ### Explication
    help_label_font = tkFont.Font(family = gg.FONT_NAME,
                                  size = eff_help_font_size)
    help_label = tk.Label(self,
                          text = gg.HELP_SYNTHESE_SORTIES,
                          justify = "left",
                          font = help_label_font)
    place_bellow(synthese_analysis_label,
                 help_label)

    ### Bouton pour lancer l'analyse de la synthese des activités
    synthese_analysis_launch_font = tkFont.Font(family = gg.FONT_NAME,
                                                size = eff_launch_font_size)
    synthese_analysis_launch_button = tk.Button(self,
                                                text = gg.BUTT_SYNTHESE_SORTIES,
                                                font = synthese_analysis_launch_font,
                                                command = lambda: _launch_synthese_analysis())
    place_bellow(help_label,
                 synthese_analysis_launch_button,
                 dx = launch_dx_px,
                 dy = launch_dy_px)
                 
    ### Choix de la méthode
    list_methods = ['activités', 'jours']
    default_methods = list_methods[0]
    variable_methods = tk.StringVar(self)
    variable_methods.set(default_methods)
    OptionButton_methods = tk.OptionMenu(self,
                                         variable_methods,
                                         *list_methods)
    OptionButton_methods.config(font = self.font_OptionButton_years)
    font_Label_years = tkFont.Font(family = gg.FONT_NAME,
                                            size = eff_select_font_size,
                                            weight = 'bold')
    Label_methods = tk.Label(self,
                             text = gg.TEXT_METHOD,
                             font = self.font_Label_years)

    place_after(synthese_analysis_launch_button, Label_methods, dx=200, dy = 0)
    place_after(Label_methods, OptionButton_methods, dy = dy_year)

    ################## Analyse des sorties par adhérent

    ### Titre
    member_analysis_label_font = tkFont.Font(family = gg.FONT_NAME,
                                             size = eff_etape_font_size,
                                             weight = 'bold')
    member_analysis_label = tk.Label(self,
                                     text = gg.TEXT_MEMBER_ANALYSIS,
                                     justify = "left",
                                     font = member_analysis_label_font)
    place_bellow(synthese_analysis_launch_button,
                 member_analysis_label,
                 dx = year_analysis_label_dx_px,
                 dy = year_analysis_label_dy_px)

    ### Explication de l'étape
    help_label_font = tkFont.Font(family = gg.FONT_NAME,
                                  size = eff_help_font_size)
    help_label = tk.Label(self,
                          text = gg.HELP_MEMBER_ANALYSIS,
                          justify = "left",
                          font = help_label_font)
    place_bellow(member_analysis_label,
                 help_label)

    ### Bouton pour lancer l'analyse des sorties par adhérent
    member_analysis_launch_font = tkFont.Font(family = gg.FONT_NAME,
                                                size = eff_launch_font_size)
    member_analysis_button = tk.Button(self,
                                         text = gg.BUTT_MEMBER_ANALYSIS,
                                         font = member_analysis_launch_font,
                                         command = lambda: _launch_member_analysis(ctg_path))
    place_bellow(help_label,
                 member_analysis_button,
                 dx = launch_dx_px,
                 dy = launch_dy_px)


__all__ = ['plot_cd_evolution']

# Standard library imports
import pathlib
from pathlib import Path

# Third party imports
import xlrd
import pandas as pd
import matplotlib.pyplot as plt

def plot_cd_evolution(ctg_path:pathlib.WindowsPath)->pd.DataFrame:
    def getBGColor(book, sheet, row, col):
        xfx = sheet.cell_xf_index(row, col)
        xf = book.xf_list[xfx]
        bgx = xf.background.pattern_colour_index
        pattern_colour = book.colour_map[bgx]
    
        return pattern_colour
    
    def addlabels(x,y,z):
            for i in range(len(y)):
                plt.text(x[i],i,z[i],size=15)
    
    file = ctg_path / Path(r'REUNION AG\DATA\Commité directeur\Membre_CD_2014-2024.xls')
    workbook = xlrd.open_workbook(file, formatting_info=True)
    worksheet = workbook.sheet_by_name('CD')
    dic_col = {}
    dic_col [(255, 128, 128)]='reconduit'
    dic_col [(204, 204, 255)]= 'nouveau'
    dic_col [(192, 192, 192)]= 'non_present'
    dic_col [(0, 255, 0) ]= 'reelu'
    dic_col [(255, 0, 0)]= 'demissionnaire'
    
    nrow = worksheet.nrows
    ncolumn = worksheet.ncols
    cd = {}
    cdt = {}
    for col in range(1,ncolumn):
        year = int(worksheet.cell_value(0,col))
        n = 0
        entrant = 0
        dem = 0
        reelu = 0
        for row in range(1,nrow):
            c = getBGColor(workbook, worksheet,  row,col)
    
            if dic_col[c] in ['reconduit','nouveau','reelu']:
                n += 1
            if dic_col[c] == 'nouveau':
                entrant += 1
            if dic_col[c] == 'demissionnaire':
               dem += 1
            if dic_col[c] =='reelu':
               reelu += 1
        cd[year] = [n,entrant,dem]
        cdt[year] = [n,entrant,dem,reelu]
    for year, val in cd.items():
        if year == 2013:
            cd[year]=cd[year]+[0]
        else:
            cd[year]=cd[year]+[n_year_1-val[0] + val[1] - val[2]]
        n_year_1 = val[0]
    df = pd.DataFrame(cd)
    df = df.T
    dg = pd.DataFrame(cdt)
    dg = dg.T
    df.columns = ["# membres","# entrants","# démissions","# sortants"]
    dg.columns = ["# membres","# nouvaux entrants","# démissions","# reelus"]
    df["# démissions"] = -df["# démissions"]
    df["# sortants"] = -df["# sortants"]
    df["# membres année précédente"] = df["# membres"] - df["# entrants"]
    ax =  df.plot(
                  y=["# membres année précédente","# entrants","# démissions","# sortants"],
                  kind='barh',
                  stacked=True,color=['g','y','r','m'],
                  xlim=[-7,25],
                  figsize=(5,10))
    
    ax.grid('on', which='minor', axis='x' )
    ax.grid('on', which='major', axis='x' )
    ax.legend(bbox_to_anchor=(1.0,1.0))
    ax.tick_params(axis='x', rotation=0, labelsize=20)
    ax.tick_params(axis='y', labelsize=20)
    df['fac renouvellement'] = 50*(df["# entrants"]-df["# sortants"]-df["# démissions"]) /df["# membres"]
    addlabels(df["# membres"].tolist(),
               df.index.tolist(),
               [f' {round(x,1)} %' for x in df['fac renouvellement'].tolist()])

    return dg



#!/usr/bin/env python

from setuptools import setup, find_packages
from os import path

# read the contents of your README file
this_directory = path.abspath(path.dirname(__file__))
with open(path.join(this_directory, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

install_requires = open(path.join(this_directory, 'requirements.txt'), encoding='utf-8').read().strip().split('\n') 

# This setup is suitable for "python setup.py develop".

setup(name='ctg',
      version='1.1.0',
      description='A toolbox for ctg statistics analysis',
      long_description=long_description,
      long_description_content_type='text/markdown',
      include_package_data = True,
      license = 'MIT',
      classifiers = [
        'Development Status :: 4 - Beta',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3.9',
        'Topic :: Scientific/Engineering :: Information Analysis',
        'Operating System :: OS Independent',
        'Intended Audience :: Science/Research'
        ],
      keywords = 'ffvelo,ctg',
      install_requires = install_requires,
      author= 'f. bertin',
      author_email= 'francois.bertin7@wanadoo.fr',
      url= 'https://github.com/Bertin-fap/ctgutils ',
      packages=['ctg\ctgfuncts','ctg\ctggui'],
      )



__author__  = 'BiblioMeter team'
__license__ = 'MIT'
__version__ = '1.1.3'

from ctg.ctggui.guiglobals import *
from ctg.ctggui.guitools import *
from ctg.ctggui.pageclasses import *
from ctg.ctggui.page_effectif import *
from ctg.ctggui.pagesorties import *
from ctg.ctggui.pagesynthese import *
from ctg.ctggui.pagetendance import *
from ctg.ctggui.pagedivers import *



__all__ = ['create_effectif_analysis']

# Standard library imports
import os
import shutil
import tkinter as tk
from tkinter import font as tkFont
from tkinter import filedialog
from tkinter import messagebox
from pathlib import Path

# 3rd party imports
import pandas as pd

# Local imports
import ctg.ctggui.guiglobals as gg
from ctg.ctggui.guitools import encadre_RL
from ctg.ctggui.guitools import font_size
from ctg.ctggui.guitools import mm_to_px
from ctg.ctggui.guitools import place_after
from ctg.ctggui.guitools import place_bellow
from ctg.ctggui.guitools import last_available_years
from ctg.ctgfuncts.ctg_classes import EffectifCtg
from ctg.ctgfuncts.ctg_effectif import evolution_age_median
from ctg.ctgfuncts.ctg_effectif import evolution_effectif
from ctg.ctgfuncts.ctg_effectif import plot_rebond

def _launch_year_analysis(ctg_path, year_select):
    """
    """

    info_title = "- Information -"
    info_text  = f"L'analyse des mots clefs a été effectuée pour l'année {year_select}."
    info_text += f"\nLes fichiers obtenus ont été créés dans le dossier :"
    info_text += f"\n\n'{year_analysis_folder_path}' "
    messagebox.showinfo(info_title, info_text)



def _launch_effectif_year_analysis(ctg_path,
                        year_select):
    """
    """

    effectif = EffectifCtg(year_select,ctg_path)
    effectif.stat()
    effectif.plot_histo()

    #info_title = "- Information -"
    #info_text  = f"L'analyse des IFs a été effectuée pour l'année {year_select} "
    #info_text += f"avec les valeurs {analysis_if}. "
    #info_text += f"\n\nLes fichiers obtenus ont été créés dans le dossier :"
    #info_text += f"\n\n''."
    #info_text += f"\n\nLa base de donnée des indicateurs respective de l'Institut "
    #info_text += f"et de chaque département a été mise à jour "
    #info_text += f"avec les résultats de cette analyse et se trouve dans le dossier :"
    #info_text += f"\n\n''."
    #messagebox.showinfo(info_title, info_text)

def _launch_effectif_analysis(ctg_path):

    evolution_effectif(ctg_path)
    
def _launch_rebond_analysis(ctg_path):

    plot_rebond(ctg_path)


def _launch_age_analysis(ctg_path):

    evolution_age_median(ctg_path)

def create_effectif_analysis(self, master, page_name, institute, ctg_path):

    """
    Description : function working as a bridge between the BiblioMeter
    App and the functionalities needed for the use of the app

    Uses the following globals :
    - DIC_OUT_PARSING
    - FOLDER_NAMES

    Args: takes only self and ctg_path as arguments.
    self is the instense in which PageThree will be created
    ctg_path is a type Path, and is the path to where the folders
    organised in a very specific way are stored

    Returns : nothing, it create the page in self
    """

    # Internal functions
    def _launch_effectif_year_analysis_try():
        # Getting year selection
        year_select =  variable_years.get()

        _launch_effectif_year_analysis(ctg_path,
                            year_select)
        return

    from ctg.ctggui.pageclasses import AppMain

    # Setting effective font sizes and positions (numbers are reference values)
    eff_etape_font_size      = font_size(gg.REF_ETAPE_FONT_SIZE,   AppMain.width_sf_min)
    eff_launch_font_size     = font_size(gg.REF_ETAPE_FONT_SIZE-1, AppMain.width_sf_min)
    eff_help_font_size       = font_size(gg.REF_ETAPE_FONT_SIZE-2, AppMain.width_sf_min)
    eff_select_font_size     = font_size(gg.REF_ETAPE_FONT_SIZE, AppMain.width_sf_min)
    eff_buttons_font_size    = font_size(gg.REF_ETAPE_FONT_SIZE-3, AppMain.width_sf_min)

    if_analysis_x_pos_px     = mm_to_px(10 * AppMain.width_sf_mm,  gg.PPI)
    if_analysis_y_pos_px     = mm_to_px(40 * AppMain.height_sf_mm, gg.PPI)
    year_analysis_label_dx_px  = mm_to_px( 0 * AppMain.width_sf_mm,  gg.PPI)
    year_analysis_label_dy_px  = mm_to_px(15 * AppMain.height_sf_mm, gg.PPI)
    launch_dx_px             = mm_to_px( 0 * AppMain.width_sf_mm,  gg.PPI)
    launch_dy_px             = mm_to_px( 5 * AppMain.height_sf_mm, gg.PPI)

    year_button_x_pos        = mm_to_px(gg.REF_YEAR_BUT_POS_X_MM * AppMain.width_sf_mm,  gg.PPI)
    year_button_y_pos        = mm_to_px(gg.REF_YEAR_BUT_POS_Y_MM * AppMain.height_sf_mm, gg.PPI)
    dy_year                  = -6
    ds_year                  = 5

    # Setting common attributs
    etape_label_format = 'left'
    etape_underline    = -1

    ### Choix de l'année
    list_years = last_available_years(ctg_path,year_number=2000)[1:]
    default_year = list_years[-1]
    variable_years = tk.StringVar(self)
    variable_years.set(default_year)

        # Création de l'option button des années
    self.font_OptionButton_years = tkFont.Font(family = gg.FONT_NAME,
                                               size = eff_buttons_font_size)
    self.OptionButton_years = tk.OptionMenu(self,
                                            variable_years,
                                            *list_years)
    self.OptionButton_years.config(font = self.font_OptionButton_years)

        # Création du label
    self.font_Label_years = tkFont.Font(family = gg.FONT_NAME,
                                        size = eff_select_font_size,
                                        weight = 'bold')
    self.Label_years = tk.Label(self,
                                text = gg.TEXT_YEAR_PI,
                                font = self.font_Label_years)
    self.Label_years.place(x = year_button_x_pos, y = year_button_y_pos)

    place_after(self.Label_years, self.OptionButton_years, dy = dy_year)

    ################## Analyse de l'effectif annuel

    ### Titre
    if_analysis_font = tkFont.Font(family = gg.FONT_NAME,
                                   size = eff_etape_font_size,
                                   weight = 'bold')
    if_analysis_label = tk.Label(self,
                                 text = gg.BUTT_EFFECTIF_ANALYSIS,
                                 justify = etape_label_format,
                                 font = if_analysis_font,
                                 underline = etape_underline)

    if_analysis_label.place(x = if_analysis_x_pos_px,
                            y = if_analysis_y_pos_px)

    ### Explication
    help_label_font = tkFont.Font(family = gg.FONT_NAME,
                                  size = eff_help_font_size)
    help_label = tk.Label(self,
                          text = gg.HELP_EFFECTIF,
                          justify = "left",
                          font = help_label_font)
    place_bellow(if_analysis_label,
                 help_label)

    ### Bouton pour lancer l'analyse de l'effecif annuel
    if_analysis_launch_font = tkFont.Font(family = gg.FONT_NAME,
                                          size = eff_launch_font_size)
    if_analysis_launch_button = tk.Button(self,
                                          text = gg.BUTT_EFFECTIF_ANALYSIS,
                                          font = if_analysis_launch_font,
                                          command = lambda: _launch_effectif_year_analysis_try())
    place_bellow(help_label,
                 if_analysis_launch_button,
                 dx = launch_dx_px,
                 dy = launch_dy_px)

    ################## Analyse de l'évolution de l'effectif

    ### Titre
    effectif_analysis_label_font = tkFont.Font(family = gg.FONT_NAME,
                                        size = eff_etape_font_size,
                                        weight = 'bold')
    effectif_analysis_label = tk.Label(self,
                               text = gg.TEXT_EVOLUTION_EFFECTIF,
                               justify = "left",
                               font = effectif_analysis_label_font)
    place_bellow(if_analysis_launch_button,
                 effectif_analysis_label,
                 dx = year_analysis_label_dx_px,
                 dy = year_analysis_label_dy_px)

    ### Explication de l'étape
    help_label_font = tkFont.Font(family = gg.FONT_NAME,
                               size = eff_help_font_size)
    help_label = tk.Label(self,
                          text = gg.HELP_EVOLUTION_EFFECTIF,
                          justify = "left",
                          font = help_label_font)
    place_bellow(effectif_analysis_label,
                 help_label)

    ### Bouton pour lancer l'analyse de l'effectif
    effectif_analysis_launch_font = tkFont.Font(family = gg.FONT_NAME,
                                                size = eff_launch_font_size)
    effectif_analysis_button = tk.Button(self,
                                         text = gg.BUTT_EVOLUTION_EFFECTIF,
                                         font = effectif_analysis_launch_font,
                                         command = lambda: _launch_effectif_analysis(ctg_path))
    place_bellow(help_label,
                 effectif_analysis_button,
                 dx = launch_dx_px,
                 dy = launch_dy_px)
                 
    rebond_analysis_button = tk.Button(self,
                                       text = gg.BUTT_REBOND,
                                       font = effectif_analysis_launch_font,
                                       command = lambda: _launch_rebond_analysis(ctg_path))
    place_after( effectif_analysis_button,
                 rebond_analysis_button,
                 dx = 100,
                 dy = 0)

################## Analyse de l'évolution de la moyenne d'âge

    ### Titre
    age_analysis_label_font = tkFont.Font(family = gg.FONT_NAME,
                                          size = eff_etape_font_size,
                                          weight = 'bold')
    age_analysis_label = tk.Label(self,
                                  text = gg.TEXT_AGE_ANALYSIS,
                                  justify = "left",
                                  font = age_analysis_label_font)
    place_bellow(effectif_analysis_button,
                 age_analysis_label,
                 dx = year_analysis_label_dx_px,
                 dy = year_analysis_label_dy_px)

    ### Explication de l'étape
    help_label_font = tkFont.Font(family = gg.FONT_NAME,
                               size = eff_help_font_size)
    help_label_age = tk.Label(self,
                              text = gg.HELP_AGE_ANALYSIS,
                              justify = "left",
                              font = help_label_font)
    place_bellow(age_analysis_label,
                 help_label_age)

    ### Bouton pour lancer l'analyse des mots clefs
    age_analysis_launch_font = tkFont.Font(family = gg.FONT_NAME,
                                         size = eff_launch_font_size)
    age_analysis_button = tk.Button(self,
                                    text = gg.BUTT_AGE_ANALYSIS,
                                    font = age_analysis_launch_font,
                                    command = lambda: _launch_age_analysis(ctg_path))
    place_bellow(help_label_age,
                 age_analysis_button,
                 dx = launch_dx_px,
                 dy = launch_dy_px)